"""Instrution for hybrid types of machine.

E.g. the BTC machine works mostly as a stack, but also allow random memory access.
"""
from typing import Any, Type
from symbolic_execution.hardware.interfaces import (
    IMachine,
    IRAMBasedMachine,
    IStackBasedMachine,
)
from .interfaces import IInstruction


class PickInstruction(IInstruction):
    """Read the data x hops from the top and push onto the stack."""

    def __init__(self, pc: int, **args):
        """Init the instruction."""
        super().__init__(pc, **args)

    def run(self, machine: Type[IMachine]):
        """Return the value in memory at position {pos} along with its size."""
        if not isinstance(machine, IStackBasedMachine) and not isinstance(
            machine, IRAMBasedMachine
        ):
            raise TypeError(
                "This instruction needs to run on a machine that is both a stack based "
                "and a ram based machine."
            )

        super().run(machine)

        n = machine.pop_from_stack()
        elt_n = machine.read(n)
        machine.push_onto_stack(elt_n)

    def __repr__(self) -> str:
        """Return the string representation of the instruction."""
        return f"{super().__repr__()}(top_stack_elt)"
